# -*- coding: utf-8 -*-

import werkzeug

from odoo.addons.openerp_enterprise.models.gandi_api import GandiError
from odoo.addons.saas_worker.compat import route, http, request
from odoo.addons.saas_worker.util import from_role, SupportLevel as SL
from odoo.exceptions import ValidationError


class DNSController(http.Controller):
    @route('/dns/status', auth='user', type='json', readonly=True)
    def dns_status(self, fqdn):
        try:
            status = request.env['openerp.enterprise.domain']._status(fqdn)
            return {'status': status}
        except Exception:
            return {'status': 'error'}

    @route('/dns/search', auth='user', type='json', readonly=True)
    def dns_search(self, search_term):
        results = request.env['openerp.enterprise.domain']._search_domain(search_term)
        return results

    @route('/dns/verify-custom-domain', auth='user', type='json', readonly=True)
    def dns_verify_custom_domain(self, domain, db):
        returncode = request.env['openerp.enterprise.domain']._verify_custom_domain(domain, db)
        return {'code': returncode}

    @route('/dns/register/prefill', auth='user', type='json', readonly=True)
    def dns_register_prefill(self):
        partner_id = request.env.user.partner_id
        partner = partner_id.read(['street', 'zip', 'city', 'email', 'phone'], load=None)
        partner = partner[0]

        if partner_id.country_id:
            partner['country_code'] = partner_id.country_id.code
        else:
            partner['country_code'] = request.geoip.country_code

        return partner

    @route('/dns/register', auth='user', type='json')
    def domain_register(self, database_id, fqdn, contact_type, fname, lname, email, phone, country, address, city, **kwargs):
        try:
            Domain = request.env['openerp.enterprise.domain']
            Domain._register_domain(database_id, fqdn, int(contact_type), fname, lname, email, phone, country, address, city, **kwargs)
            return {'success': True}
        except GandiError as e:
            return {'errors': [err['description'] for err in e.errors]}
        except ValidationError as e:
            return {'errors': [e.args[0]]}
        except Exception as e:
            return {'errors': [str(e)]}

    @route('/dns/domains', auth='user', type='json', readonly=True)
    def list_domains(self, db_id, **kwargs):
        domains = request.env['openerp.enterprise.domain']._get_user_domains(db_id)
        return [{
            'fqdn': d.fqdn,
            'expiration_date': d.expiration_date,
            'subscription_code': d.subscription_id.client_order_ref,
            'state': d.state,
        } for d in domains]

    @route('/dns/domains/<string:fqdn>/contacts', auth='user', type='json', readonly=True)
    def list_domain_contacts(self, fqdn):
        try:
            domain = request.env['openerp.enterprise.domain']._get_domain(fqdn)
            contacts = domain._get_contacts()
            return contacts
        except Exception as e:
            return {'error': str(e)}

    @route('/dns/domains/<string:fqdn>/records', auth='user', type='json', readonly=True)
    def list_dns_records(self, fqdn):
        try:
            domain = request.env['openerp.enterprise.domain']._get_domain(fqdn)
            if domain.state == 'integrated':
                records = domain._get_dns_records()
            else:
                records = []

            readonly = False

            try:
                self._check_domain_name_support_access()
            except werkzeug.exceptions.Forbidden:
                readonly = True

            return {
                'records': records,
                'readonly': readonly,
                'state': domain.state,
            }
        except Exception as e:
            return {'error': str(e)}

    @route('/dns/domains/<string:fqdn>/records/<string:rtype>/<string:name>', auth='user', type='json', readonly=True)
    def get_record_content(self, fqdn, rtype, name):
        try:
            domain = request.env['openerp.enterprise.domain']._get_domain(fqdn)
            if domain.state != 'integrated':
                return {'error': 'Your domain name is being set up. The process may take a few minutes.'}

            content = domain._get_record_content(rtype, name)
            return {
                'content': content,
            }
        except Exception as e:
            return {'error': str(e)}

    @route('/dns/domains/<string:fqdn>/records/put', auth='user', type='json')
    def put_dns_record(self, fqdn, rtype, name, content, **kwargs):
        try:
            self._check_domain_name_support_access()
            domain = request.env['openerp.enterprise.domain']._get_domain(fqdn)
            if domain.state != 'integrated':
                return {'error': 'Your domain name is being set up. The process may take a few minutes.'}

            domain._put_dns_record(rtype, name, content, **kwargs)
            return {'success': True}
        except Exception as e:
            return {'error': str(e)}

    def _check_domain_name_support_access(self):
        """Restrict access to functional support."""
        impostor_info = request.session.impostor_info
        if impostor_info and impostor_info.get('support_level', SL.CUSTOMER) < SL.FUNCTIONAL_SUPPORT:
            raise werkzeug.exceptions.Forbidden()

    @route('/dns/registration-status', type='json', auth='user', readonly=True)
    def dns_registration_status(self, db_id):
        db = request.env['openerp.enterprise.database'].sudo().browse(db_id)
        status = request.env['openerp.enterprise.domain']._registration_status(db)
        return status

    @from_role('trial')
    @route('/dns/info', type='json', auth='none', readonly=True)
    def dns_info(self, db_uuid):
        domain = request.env['openerp.enterprise.domain'].sudo().search([
            ('database_id.db_uuid', '=', db_uuid),
        ], limit=1)

        if not domain:
            return {}

        return {
            'fqdn': domain.fqdn,
        }
